import functools
import math
import operator
from typing import Any, List, Iterable, Union


def do_flatten_list(list_of_list: List[List[Any]])-> List[Any]:
    """Flatten a list of list to a list

    :param list_of_list: list to flatten
    :type list_of_list: List[List[Any]]
    :return: flatten list
    :rtype: List[Any]
    """
    return functools.reduce(operator.iconcat, list_of_list, [])

def map_many(functions: List, iterable:List[Any]) -> List[Any]:
    return list(functools.reduce(lambda x, y: map(y, x), functions, iterable))

def get(items: List[Any], idx: int, default: Any="") -> Any:
    if idx < 0:
        return default
    try:
        return items[idx]
    except IndexError:
        return default
    
def divide_into_chunks(lst: List[Any], num_chunks: int) -> List[List[Any]]:
    num_of_items_in_chunk = math.ceil(len(lst) / num_chunks)
    return [lst[i:i+num_of_items_in_chunk] for i in range(0, len(lst), num_of_items_in_chunk)]

def chunks(iterator: Union[Iterable[Any], List[Any]], chunk_size: int) -> Iterable[List[Any]]:
    """Yield successive n-sized chunks from iterator."""
    chunk = []
    for x in iterator:
        chunk.append(x)
        if len(chunk) >= chunk_size:
            yield chunk
            chunk = []
    if len(chunk) > 0:
        yield chunk


import socket


def get_local_ip() -> str:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    try:
        # doesn't even have to be reachable
        s.connect(("10.254.254.254", 1))
        IP = s.getsockname()[0]
    except Exception:
        IP = "127.0.0.1"
    finally:
        s.close()
    return IP


def get_host_name() -> str:
    return socket.gethostname()



import sys

sys_version = sys.version_info
__version__ = "0.0.1"
__all__ = [
    "concurrent",
    "file",
    "io",
    "list",
    "logger",
    "metrics",
    "misc",
    "ml",
    "pattern",
    "pg",
    "slack",
    "socket",
    "sql",
    "string",
    "tensor",
    "testing",
    "time",
]

if sys_version[0] == 3 and sys_version[1] > 7:
    import hkkang_utils.concurrent
import hkkang_utils.file
import hkkang_utils.io
import hkkang_utils.list
import hkkang_utils.logger
import hkkang_utils.metrics
import hkkang_utils.misc
import hkkang_utils.ml
import hkkang_utils.pattern
import hkkang_utils.pg
import hkkang_utils.slack
import hkkang_utils.socket
import hkkang_utils.sql
import hkkang_utils.string
import hkkang_utils.tensor
import hkkang_utils.testing
import hkkang_utils.time
